@@ -136,10 +136,13 @@ impl Timer {
         }
 
         let ffi_cb = cb.ffi_cb;
-        let ffi_data = match cb.variant {
-            CallbackVariant::Plain(cb) => cb as *mut ffi::c_void,
-            CallbackVariant::PlainUnsafe(cb) => cb as *mut ffi::c_void,
-            CallbackVariant::Closure(ref cb) => &*cb as *const _ as *mut ffi::c_void,
+        let (data, ffi_data) = match cb.variant {
+            CallbackVariant::Plain(cb) => (BoxFnPtr(0), cb as *mut ffi::c_void),
+            CallbackVariant::PlainUnsafe(cb) => (BoxFnPtr(0), cb as *mut ffi::c_void),
+            CallbackVariant::Closure(cb) => unsafe {
+                let raw = Box::into_raw(cb);
+                (BoxFnPtr(mem::transmute(raw)), raw as *mut ffi::c_void)
+            },
         };
 
         let handle = unsafe {
@@ -150,13 +153,8 @@ impl Timer {
             Ok(_) => match handle.is_null() {
                 true => false,
                 false => {
-                    match cb.variant {
-                        CallbackVariant::Closure(cb) => {
-                            //safe because we can never reach here once `handle.is_null() != true`
-                            self.data.set(cb.into())
-                        },
-                        _ => (),
-                    }
+                    //safe because we can never reach here once `handle.is_null() != true`
+                    self.data.set(data);
                     true
                 },
             },
@@ -174,10 +172,13 @@ impl Timer {
     ///On failure, returns `None`
     pub fn new(cb: Callback) -> Option<Self> {
         let ffi_cb = cb.ffi_cb;
-        let ffi_data = match cb.variant {
-            CallbackVariant::Plain(cb) => cb as *mut ffi::c_void,
-            CallbackVariant::PlainUnsafe(cb) => cb as *mut ffi::c_void,
-            CallbackVariant::Closure(ref cb) => &*cb as *const _ as *mut ffi::c_void,
+        let (data, ffi_data) = match cb.variant {
+            CallbackVariant::Plain(cb) => (BoxFnPtr(0), cb as *mut ffi::c_void),
+            CallbackVariant::PlainUnsafe(cb) => (BoxFnPtr(0), cb as *mut ffi::c_void),
+            CallbackVariant::Closure(cb) => unsafe {
+                let raw = Box::into_raw(cb);
+                (BoxFnPtr(mem::transmute(raw)), raw as *mut ffi::c_void)
+            },
         };
 
         let handle = unsafe {
@@ -188,11 +189,6 @@ impl Timer {
             return None;
         }
 
-        let data = match cb.variant {
-            CallbackVariant::Closure(cb) => cb.into(),
-            _ => BoxFnPtr::new(),
-        };
-
         Some(Self {
             inner: AtomicPtr::new(handle),
             data: Cell::new(data),
